Index: src/Fl_Input_.cxx
===================================================================
--- src/Fl_Input_.cxx	(revision 10022)
+++ src/Fl_Input_.cxx	(working copy)
@@ -428,37 +428,45 @@
 /**
   Finds the end of a word.
 
-  This call calculates the end of a word based on the given 
-  index \p i. Calling this function repeatedly will move
-  forwards to the end of the text.
- 
+  Return the index after the last byte of a space-separated word.  This
+  first skips spaces, and then non-spaces, so if you call it repeatedly
+  you will move forwards to the end of the text.
+
+  Note that this is inconsistent with line_end().
+
   \param [in] i starting index for the search
   \return end of the word
 */
 int Fl_Input_::word_end(int i) const {
-  if (input_type() == FL_SECRET_INPUT) return size();
-  //while (i < size() && !isword(index(i))) i++;
-  while (i < size() && !isword(index(i))) i++;
-  while (i < size() && isword(index(i))) i++;
+  if (input_type() == FL_SECRET_INPUT)
+      return size();
+  while (i < size() && !isword(index(i)))
+      i++;
+  while (i < size() && isword(index(i)))
+      i++;
   return i;
 }
 
 /**
   Finds the start of a word.
 
-  This call calculates the start of a word based on the given 
-  index \p i. Calling this function repeatedly will move
-  backwards to the beginning of the text.
- 
+  Return the index of the first byte of a space-separated word.
+  If the index is already at the beginning of the word, it will find the
+  beginning of the previous word, so if you call it repeatedly you will
+  move backwards to the beginning of the text.
+
+  Note that this is inconsistent with line_start().
+
   \param [in] i starting index for the search
-  \return start of the word
+  \return start of the word, or previous word
 */
 int Fl_Input_::word_start(int i) const {
-  if (input_type() == FL_SECRET_INPUT) return 0;
-//   if (i >= size() || !isword(index(i)))
-//     while (i > 0 && !isword(index(i-1))) i--;
-  while (i > 0 && !isword(index(i-1))) i--;
-  while (i > 0 && isword(index(i-1))) i--;
+  if (input_type() == FL_SECRET_INPUT)
+      return 0;
+  while (i > 0 && !isword(index(i-1)))
+      i--;
+  while (i > 0 && isword(index(i-1)))
+      i--;
   return i;
 }
 
@@ -518,6 +526,18 @@
   } else return j;
 }
 
+static int strict_word_start(const char *s, int i) {
+  while (i > 0 && !isspace(s[i-1]))
+    i--;
+  return i;
+}
+
+static int strict_word_end(const char *s, int len, int i) {
+  while (i < len && !isspace(s[i]))
+    i++;
+  return i;
+}
+
 /** 
   Handles mouse clicks and mouse moves.
   \todo Add comment and parameters
@@ -564,23 +584,23 @@
   if (Fl::event_clicks()) {
     if (newpos >= newmark) {
       if (newpos == newmark) {
-	if (newpos < size()) newpos++;
-	else newmark--;
+        if (newpos < size()) newpos++;
+        else newmark--;
       }
       if (Fl::event_clicks() > 1) {
-	newpos = line_end(newpos);
-	newmark = line_start(newmark);
+        newpos = line_end(newpos);
+        newmark = line_start(newmark);
       } else {
-	newpos = word_end(newpos);
-	newmark = word_start(newmark);
+        newpos = strict_word_end(value_, size_, newpos);
+        newmark = strict_word_start(value_, newmark);
       }
     } else {
       if (Fl::event_clicks() > 1) {
-	newpos = line_start(newpos);
-	newmark = line_end(newmark);
+        newpos = line_start(newpos);
+        newmark = line_end(newmark);
       } else {
-	newpos = word_start(newpos);
-	newmark = word_end(newmark);
+        newpos = strict_word_start(value_, newpos);
+        newmark = strict_word_end(value_, size_, newmark);
       }
     }
     // if the multiple click does not increase the selection, revert
